#region Copyright
/*
 * Copyright 2005-2008 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Reflection;
using Seasar.Extension.ADO;
using Seasar.Framework.Aop;
using Seasar.Framework.Aop.Impl;
using Seasar.Quill.Attrs;
using Seasar.Quill.Dao;
using Seasar.Quill.Dao.Interceptor;
using Seasar.Quill.Database.DataSource.Impl;
using Seasar.Quill.Database.Tx;
using Seasar.Quill.Exception;
using Seasar.Quill.Util;

namespace Seasar.Quill
{
    /// <summary>
    /// Aspect`\zNX
    /// </summary>
    /// <remarks>
    /// <para>
    /// AspectKpꍇ̓C^[tF[XENX̓\bh
    /// <see cref="Seasar.Quill.Attrs.AspectAttribute"/>()
    /// ݒ肳ĂKvB
    /// </para>
    /// <para>
    /// S2Daő@\(diconȂ)gpɂ̓C^[tF[XENX̓\bh
    /// <see cref="Seasar.Quill.Attrs.S2DaoAttribute"/>()
    /// ݒ肳ĂKvB
    /// </para>
    /// <para>
    /// Transaction(diconȂ)gpɂ̓C^[tF[XENX̓\bh
    /// <see cref="Seasar.Quill.Attrs.TransactionAttribute"/>()
    /// ݒ肳ĂKvB
    /// </para>
    /// </remarks>
    public class AspectBuilder
    {
        // AspectBuilderŎgpQuillContainer
        // (Interceptor擾ׂɎgp)
        protected QuillContainer container;

        /// <summary>
        /// AspectBuilder邽߂̃RXgN^
        /// </summary>
        /// <param name="container">AspectBuilderŎgpQuillContainer</param>
        public AspectBuilder(QuillContainer container)
        {
            // AspectBuilderŎgp邽߂QuillContainer
            // RXgN^󂯎
            this.container = container;
        }

        /// <summary>
        /// w肳ꂽ<code>type</code>̑mF
        /// Aspect`̔z쐬
        /// </summary>
        /// <param name="type">Aspect`mFType</param>
        /// <returns>쐬ꂽAspect`̔z</returns>
        public virtual IAspect[] CreateAspects(Type type)
        {
            // Aspect̃Xg
            List<IAspect> aspectList = new List<IAspect>();

            // typeŐ錾Ă郁\bh擾
            MethodInfo[] methods = type.GetMethods(BindingFlags.Public | 
                BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.Static);

            CreateFromAspectAttribute(type, methods, aspectList);
            CreateFromTransactionAttribute(type, methods, aspectList);
            CreateFromS2DaoAttribute(type, methods, aspectList);

            // Aspect̔zԂ
            return aspectList.ToArray();
        }

        /// <summary>
        /// AspectAXyNg쐬
        /// </summary>
        /// <param name="targetType">AspectKp郁\bh^</param>
        /// <param name="methods">AspectKp郁\bh</param>
        /// <param name="aspectList">KpAspect̃Xg</param>
        protected virtual void CreateFromAspectAttribute(
            Type targetType, MethodInfo[] methods, List<IAspect> aspectList)
        {
            // TypeɎw肳ꂽAspectw肷鑮擾
            AspectAttribute[] attrsByType = AttributeUtil.GetAspectAttrs(targetType);

            // TypeɎw肳ĂAspecťAAspectXgɒǉ
            foreach (AspectAttribute attrByType in attrsByType)
            {
                // PointcutɑSẴ\bhǉĂAspect쐬
                IAspect aspect = CreateAspect(attrByType);

                // 쐬AspectXgɒǉ
                aspectList.Add(aspect);
            }

            // MethodɎw肳ĂAspectXgɒǉ
            aspectList.AddRange(CreateAspectList(methods));
        }

        /// <summary>
        /// TransactionAXyNg쐬
        /// </summary>
        /// <param name="targetType">AspectKp郁\bh^</param>
        /// <param name="methods">AspectKp郁\bh</param>
        /// <param name="aspectList">KpAspect̃Xg</param>
        protected virtual void CreateFromTransactionAttribute(
            Type targetType, MethodInfo[] methods, List<IAspect> aspectList)
        {
            //  TypeɎw肳ꂽgUNVw肷鑮擾
            TransactionAttribute txAttrByType = AttributeUtil.GetTransactionAttr(targetType);
            if (txAttrByType != null)
            {
                IAspect txAspect = CreateTxAspect(txAttrByType);
                if (txAspect != null)
                {
                    aspectList.Add(txAspect);
                }
            }

            // MethodɎw肳ĂTransactionpAspectXgɒǉ
            aspectList.AddRange(CreateTxAspectList(methods));
        }

        /// <summary>
        /// S2DaoAXyNg쐬
        /// </summary>
        /// <param name="targetType">AspectKp郁\bh^</param>
        /// <param name="methods">AspectKp郁\bh</param>
        /// <param name="aspectList">KpAspect̃Xg</param>
        protected virtual void CreateFromS2DaoAttribute(
            Type targetType, MethodInfo[] methods, List<IAspect> aspectList)
        {
            //  TypeɎw肳ꂽDaoInterceptorw肷鑮擾
            S2DaoAttribute daoAttrByType = AttributeUtil.GetS2DaoAttr(targetType);

            if (daoAttrByType != null)
            {
                //  f[^\[XIInterceptor`Ăꍇ
                //  Aspecto^
                IAspect dataSourceSelctAspect = GetDataSourceSelectAspect(daoAttrByType, targetType);
                if (dataSourceSelctAspect != null)
                {
                    aspectList.Add(dataSourceSelctAspect);
                }

                //  S2DaoInterceptoro^
                IAspect daoAspectToClass = CreateS2DaoAspect(daoAttrByType);
                if (daoAspectToClass != null)
                {
                    aspectList.Add(daoAspectToClass);
                }
            }

            // MethodɎw肳ĂTransactionpAspectXgɒǉ
            IList<IAspect> daoAspectsToMethod = CreateS2DaoAspectList(methods);
            if (daoAspectsToMethod != null && daoAspectsToMethod.Count > 0)
            {
                aspectList.AddRange(daoAspectsToMethod);
            }
        }

        /// <summary>
        /// f[^\[XIInterceptor̎擾
        /// </summary>
        /// <param name="daoAttr">S2Dao</param>
        /// <param name="targetMember">InterceptorΏ</param>
        /// <returns>f[^\[XIInterceptor</returns>
        protected virtual IMethodInterceptor GetDataSourceSelectInterceptor(S2DaoAttribute daoAttr,
            MemberInfo targetMember)
        {
            DataSourceSelectInterceptor dsInterceptor = null;
            Type daoSettingType = daoAttr.DaoSettingType;
            if (daoSettingType != null)
            {
                IDaoSetting daoSetting = (IDaoSetting)ComponentUtil.GetComponent(container, daoSettingType);

                string dataSourceName = daoSetting.DataSourceName;
                //  f[^\[X`ĂInterceptorĕԂ
                if (string.IsNullOrEmpty(dataSourceName) == false)
                {
                    dsInterceptor = (DataSourceSelectInterceptor)ComponentUtil.GetComponent(
                        container, typeof(DataSourceSelectInterceptor));

                    // oƃf[^\[XΉt
                    dsInterceptor.DaoDataSourceMap[targetMember] = dataSourceName;

                    //  f[^\[Xݒ̏ꍇ̓Zbg
                    if (dsInterceptor.DataSourceProxy == null)
                    {
                        SelectableDataSourceProxyWithDictionary ds =
                            (SelectableDataSourceProxyWithDictionary)ComponentUtil.GetComponent(
                            container, typeof(SelectableDataSourceProxyWithDictionary));
                        dsInterceptor.DataSourceProxy = ds;
                    }
                }
            }
            return dsInterceptor;
        }

        /// <summary>
        /// f[^\[XIAspect̎擾
        /// </summary>
        /// <param name="daoAttr">S2Dao</param>
        /// <param name="targetType">AspectΏ</param>
        /// <returns>f[^\[XIAspect</returns>
        protected virtual IAspect GetDataSourceSelectAspect(S2DaoAttribute daoAttr, MemberInfo targetMember)
        {
            IMethodInterceptor dataSourceSelectInterceptor = GetDataSourceSelectInterceptor(
                daoAttr, targetMember);

            IAspect dataSourceSelectAspect = null;
            if(dataSourceSelectInterceptor != null)
            {
                dataSourceSelectAspect = new AspectImpl(dataSourceSelectInterceptor);
            }
            return dataSourceSelectAspect;
        }

        /// <summary>
        /// w肳ꂽ\bhAspectLƂȂAspect`̃Xg쐬
        /// </summary>
        /// <param name="methods">\bh̔z</param>
        /// <returns>w肳ꂽ\bhAspectLƂȂAspect`̃Xg</returns>
        protected virtual IList<IAspect> CreateAspectList(MethodInfo[] methods)
        {
            // InterceptorpointcutƂȂ郁\bhi[
            IDictionary<IMethodInterceptor, List<string>> methodNames =
                new Dictionary<IMethodInterceptor, List<string>>();

            // InterceptorpointcutƂȂ郁\bhi[
            foreach (MethodInfo method in methods)
            {
                // AspectmFPointcut쐬ׂ̃\bhǉ
                AddMethodNamesForPointcut(methodNames, method);
            }

            // Aspect̃Xg
            List<IAspect> aspectList = new List<IAspect>();

            // InterceptořAAspect쐬
            foreach (IMethodInterceptor interceptor in methodNames.Keys)
            {
                // Interceptorƃ\bh̔z񂩂Aspect쐬
                IAspect aspect = CreateAspect(
                    interceptor, methodNames[interceptor].ToArray());

                // Aspect̃Xgɒǉ
                aspectList.Add(aspect);
            }

            // Aspect̃XgԂ
            return aspectList.ToArray();
        }

        /// <summary>
        /// SẴ\bhAspectLƂȂAspect`쐬
        /// </summary>
        /// <param name="aspectAttr">Aspectݒ肷鑮</param>
        /// <returns>SẴ\bhAspectLƂȂAspect`</returns>
        protected virtual IAspect CreateAspect(AspectAttribute aspectAttr)
        {
            // Interceptor쐬
            IMethodInterceptor interceptor = GetMethodInterceptor(aspectAttr);

            // InterceptorAspect쐬
            // (Pointcut͎w肵Ȃ̂őSẴ\bhΏۂƂȂ)
            IAspect aspect = new AspectImpl(interceptor);

            // AspectԂ
            return aspect;
        }

        /// <summary>
        /// C^[Zv^[ƃ\bhw肵Aspect`쐬
        /// </summary>
        /// <param name="interceptor">C^[Zv^[</param>
        /// <param name="methodNames">AspectKp郁\bh̔z</param>
        /// <returns>w肳ꂽ\bhAspectLƂȂAspect`</returns>
        protected virtual IAspect CreateAspect(
            IMethodInterceptor interceptor, string[] methodNames)
        {
            // Pointcut쐬
            IPointcut pointcut = new PointcutImpl(methodNames);

            // InterceptorPointcutAspect쐬
            IAspect aspect = new AspectImpl(interceptor, pointcut);

            // AspectԂ
            return aspect;
        }

        /// <summary>
        /// AspectmFPointcut쐬ׂ̃\bhǉ
        /// </summary>
        /// <param name="methods">ΏۃNX̃\bh</param>
        /// <returns>KpAspect̃Xg</returns>
        protected virtual IAspect[] CreateTxAspectList(MethodInfo[] methods)
        {
            IDictionary<IMethodInterceptor, List<string>> methodNames =
                new Dictionary<IMethodInterceptor, List<string>>();
            foreach ( MethodInfo method in methods )
            {
                TransactionAttribute txAttr = AttributeUtil.GetTransactionAttrByMethod(method);
                if ( txAttr != null )
                {
                    AddMethodNamesForTxPointcut(methodNames, method.Name, txAttr);
                }
            }

            // Aspect̃Xg
            List<IAspect> txList = new List<IAspect>();

            // InterceptořAAspect쐬
            foreach ( IMethodInterceptor interceptor in methodNames.Keys )
            {
                // Interceptorƃ\bh̔z񂩂Aspect쐬
                IAspect aspect = CreateAspect(
                    interceptor, methodNames[interceptor].ToArray());

                // Aspect̃Xgɒǉ
                txList.Add(aspect);
            }

            // Aspect̃XgԂ
            return txList.ToArray();
        }

        /// <summary>
        /// SẴ\bhAspectLƂȂAspect`쐬
        /// </summary>
        /// <param name="txAttr">Aspectݒ肷鑮</param>
        /// <returns>SẴ\bhAspectLƂȂAspect`</returns>
        protected virtual IAspect CreateTxAspect(TransactionAttribute txAttr)
        {
            // Interceptor쐬
            IMethodInterceptor interceptor = GetMethodInterceptor(txAttr);

            // InterceptorAspect쐬
            // (Pointcut͎w肵Ȃ̂őSẴ\bhΏۂƂȂ)
            IAspect aspect = new AspectImpl(interceptor);

            // AspectԂ
            return aspect;
        }

        /// <summary>
        /// \bh񂩂ǉ邽߂Aspect̃Xg쐬
        /// </summary>
        /// <param name="methods">ΏۃNX̃\bh</param>
        /// <returns>KpAspect̃Xg</returns>
        protected virtual IAspect[] CreateS2DaoAspectList(MethodInfo[] methods)
        {
            // f[^\[XIInterceptorKp\bhRNV
            IDictionary<IMethodInterceptor, List<string>> dataSourceSelectMethodNames =
                new Dictionary<IMethodInterceptor, List<string>>();
            // S2DaoInterceptorKp\bhRNV
            IDictionary<IMethodInterceptor, List<string>> daoMethodNames =
                new Dictionary<IMethodInterceptor, List<string>>();
            //  \bhIntarceptoȓΉt
            foreach (MethodInfo method in methods)
            {
                S2DaoAttribute daoAttr = AttributeUtil.GetS2DaoAttrByMethod(method);
                if (daoAttr != null)
                {
                    AddMethodNamesForDataSourceSelectPointcut(dataSourceSelectMethodNames, method, daoAttr);
                    AddMethodNamesForS2DaoPointcut(daoMethodNames, method.Name, daoAttr);
                }
            }

            // Aspect̃Xg
            List<IAspect> daoAspectList = new List<IAspect>();

            //  f[^\[XIInterceptor͐ɓo^
            foreach (IMethodInterceptor dataSourceSelectInterceptor in dataSourceSelectMethodNames.Keys)
            {
                // Interceptorƃ\bh̔z񂩂Aspect쐬
                IAspect dataSourceSelectAspect = CreateAspect(dataSourceSelectInterceptor,
                    dataSourceSelectMethodNames[dataSourceSelectInterceptor].ToArray());

                // Aspect̃Xgɒǉ
                daoAspectList.Add(dataSourceSelectAspect);
            }

            // InterceptořAAspect쐬
            foreach (IMethodInterceptor daoInterceptor in daoMethodNames.Keys)
            {
                // Interceptorƃ\bh̔z񂩂Aspect쐬
                IAspect daoAspect = CreateAspect(
                    daoInterceptor, daoMethodNames[daoInterceptor].ToArray());

                // Aspect̃Xgɒǉ
                daoAspectList.Add(daoAspect);
            }

            // Aspect̃XgԂ
            return daoAspectList.ToArray();
        }

        /// <summary>
        /// SẴ\bhAspectLƂȂAspect`쐬
        /// </summary>
        /// <param name="daoAttr">Aspectݒ肷鑮</param>
        /// <returns>SẴ\bhAspectLƂȂAspect`</returns>
        protected virtual IAspect CreateS2DaoAspect(S2DaoAttribute daoAttr)
        {
            // Interceptor쐬
            IMethodInterceptor interceptor = GetMethodInterceptor(daoAttr);

            // InterceptorAspect쐬
            // (Pointcut͎w肵Ȃ̂őSẴ\bhΏۂƂȂ)
            IAspect aspect = new AspectImpl(interceptor);

            // AspectԂ
            return aspect;
        }

        /// <summary>
        /// AspectmFPointcut쐬ׂ̃\bhǉ
        /// </summary>
        /// <param name="methodNames">
        /// InterceptorpointcutƂȂ郁\bhi[RNV
        /// </param>
        /// <param name="method">\bh</param>
        protected void AddMethodNamesForPointcut(
            IDictionary<IMethodInterceptor, List<string>> methodNames, MethodInfo method)
        {
            // \bhɎw肳ĂAspect擾
            AspectAttribute[] aspectAttrs =
                AttributeUtil.GetAspectAttrsByMethod(method);

            // AspecťAPointcut쐬ׂ̃\bhǉ
            foreach (AspectAttribute aspectAttr in aspectAttrs)
            {
                // Pointcut쐬ׂ̃\bhǉ
                AddMethodNamesForPointcut(methodNames, method.Name, aspectAttr);
            }
        }

        /// <summary>
        /// AspectmFPointcut쐬ׂ̃\bhǉ
        /// </summary>
        /// <param name="methodNames">
        /// InterceptorpointcutƂȂ郁\bhi[RNV
        /// </param>
        /// <param name="methodName">\bh</param>
        /// <param name="aspectAttr">Aspect</param>
        protected void AddMethodNamesForPointcut(
            IDictionary<IMethodInterceptor, List<string>> methodNames,
             string methodName, AspectAttribute aspectAttr)
        {
            // C^[Zv^[擾
            IMethodInterceptor interceptor = GetMethodInterceptor(aspectAttr);

            if (!methodNames.ContainsKey(interceptor))
            {
                // n߂ĂInterceptoȑꍇstring̃Xg
                methodNames.Add(interceptor, new List<string>());
            }

            // \bhǉ
            methodNames[interceptor].Add(methodName);
        }

        /// <summary>
        /// AspectmFPointcut쐬ׂ̃\bhǉ
        /// </summary>
        /// <param name="methodNames">
        /// InterceptorpointcutƂȂ郁\bhi[RNV
        /// </param>
        /// <param name="methodName">\bh</param>
        /// <param name="txAttr">Aspect</param>
        protected void AddMethodNamesForTxPointcut(
            IDictionary<IMethodInterceptor, List<string>> methodNames,
             string methodName, TransactionAttribute txAttr)
        {
            // C^[Zv^[擾
            IMethodInterceptor interceptor = GetMethodInterceptor(txAttr);

            if ( !methodNames.ContainsKey(interceptor) )
            {
                // n߂ĂInterceptoȑꍇstring̃Xg
                methodNames.Add(interceptor, new List<string>());
            }

            // \bhǉ
            methodNames[interceptor].Add(methodName);
        }

        /// <summary>
        /// S2Daõf[^\[X`mFPointcut쐬ׂ̃\bhǉ
        /// </summary>
        /// <param name="methodNames">
        /// InterceptorpointcutƂȂ郁\bhi[RNV
        /// </param>
        /// <param name="method">AspectKp郁\bh</param>
        /// <param name="daoAttr">S2Dao</param>
        protected void AddMethodNamesForDataSourceSelectPointcut(
            IDictionary<IMethodInterceptor, List<string>> methodNames, 
            MethodInfo method, S2DaoAttribute daoAttr)
        {
            //  f[^\[XIInterceptor̎擾
            IMethodInterceptor dataSourceSelectInterceptor =
                GetDataSourceSelectInterceptor(daoAttr, method);

            if (dataSourceSelectInterceptor != null)
            {
                if (!methodNames.ContainsKey(dataSourceSelectInterceptor))
                {
                    // n߂ĂInterceptoȑꍇstring̃Xg
                    methodNames.Add(dataSourceSelectInterceptor, new List<string>());
                }
                methodNames[dataSourceSelectInterceptor].Add(method.Name);
            }
        }

        /// <summary>
        /// S2DaomFPointcut쐬ׂ̃\bhǉ
        /// </summary>
        /// <param name="methodNames">
        /// InterceptorpointcutƂȂ郁\bhi[RNV
        /// </param>
        /// <param name="methodName">\bh</param>
        /// <param name="daoAttr">S2Dao</param>
        protected void AddMethodNamesForS2DaoPointcut(
            IDictionary<IMethodInterceptor, List<string>> methodNames,
             string methodName, S2DaoAttribute daoAttr)
        {
            // C^[Zv^[擾
            IMethodInterceptor interceptor = GetMethodInterceptor(daoAttr);

            if (!methodNames.ContainsKey(interceptor))
            {
                // n߂ĂInterceptoȑꍇstring̃Xg
                methodNames.Add(interceptor, new List<string>());
            }

            // \bhǉ
            methodNames[interceptor].Add(methodName);
        }

        /// <summary>
        /// AspectC^[Zv^[擾
        /// </summary>
        /// <param name="aspectAttr">Aspect</param>
        /// <returns>C^[Zv^[</returns>
        protected virtual IMethodInterceptor GetMethodInterceptor(
            AspectAttribute aspectAttr)
        {
            if (aspectAttr.InterceptorType != null)
            {
                // interceptorTypew肳Ăꍇ
                // QuillTypew肵ăC^[Zv^[擾
                return GetMethodInterceptor(aspectAttr.InterceptorType);
            }
            else if (aspectAttr.ComponentName != null)
            {
                // R|[lgw肳Ăꍇ
                // S2ContainerR|[lgw肵ăC^[Zv^[擾
                return GetMethodInterceptor(aspectAttr.ComponentName);
            }
            else
            {
                // AspectinterceptorTypecomponentNamêǂ̎w
                // ĂȂꍇ͗OX[
                throw new QuillApplicationException("EQLL0013");
            }
        }

        /// <summary>
        /// QuillTypew肵ăC^[Zv^[擾
        /// </summary>
        /// <param name="interceptorType">C^[Zv^[Type</param>
        /// <returns>C^[Zv^[</returns>
        protected virtual IMethodInterceptor GetMethodInterceptor(
            Type interceptorType)
        {
            // Interceptor̃R|[lg擾
            QuillComponent component =
                container.GetComponent(interceptorType);

            if (typeof(IMethodInterceptor).IsAssignableFrom(component.ComponentType))
            {
                // IMethodInterceptorɑłꍇInterceptorԂ
                return (IMethodInterceptor)component.GetComponentObject(interceptorType);
            }
            else
            {
                // IMethodInterceptorɑłȂꍇ͗OX[
                throw new QuillApplicationException("EQLL0012",
                    new object[] { component.ComponentType.FullName });
            }
        }

        /// <summary>
        /// S2ContainerR|[lgw肵ăC^[Zv^[擾
        /// </summary>
        /// <param name="componentName">R|[lg</param>
        /// <returns>C^[Zv^[</returns>
        protected virtual IMethodInterceptor GetMethodInterceptor(
            string componentName)
        {
            // S2ContainerR|[lg̃IuWFNg擾
            object interceptor =
                SingletonS2ContainerConnector.GetComponent(componentName);

            // C^[Zv^[Type擾
            Type type = TypeUtil.GetType(interceptor);

            if (typeof(IMethodInterceptor).IsAssignableFrom(type))
            {
                // IMethodInterceptorɑłꍇInterceptorԂ
                return (IMethodInterceptor)interceptor;
            }
            else
            {
                // IMethodInterceptorɑłȂꍇ͗OX[
                throw new QuillApplicationException("EQLL0012",
                    new object[] { type.FullName });
            }
        }

        /// <summary>
        /// TransactionC^[Zv^[擾
        /// </summary>
        /// <param name="txAttr">Transaction</param>
        /// <returns>C^[Zv^[</returns>
        protected virtual IMethodInterceptor GetMethodInterceptor(
            TransactionAttribute txAttr)
        {
            Type settingType = txAttr.TransactionSettingType;
            if (settingType == null)
            {
                // gUNVݒ肪w肳ĂȂꍇ
                // OX[
                throw new QuillApplicationException("EQLL0013");
            }

            ITransactionSetting txSetting =
                (ITransactionSetting)ComponentUtil.GetComponent(container, settingType);
            if (txSetting.IsNeedSetup())
            {
                //  DataSource̎擾
                IDataSource dataSource = (IDataSource)ComponentUtil.GetComponent(
                    container, typeof(SelectableDataSourceProxyWithDictionary));
                //  gUNV֌W̐ݒ
                txSetting.Setup(dataSource);
            }

            if (txSetting.TransactionInterceptor == null)
            {
                //  InterceptorĂȂꍇ͗OƂ
                throw new QuillApplicationException("EQLL0024");
            }
            return txSetting.TransactionInterceptor;
        }

        /// <summary>
        /// S2DaoC^[Zv^[擾
        /// </summary>
        /// <param name="daoAttr">S2Dao</param>
        /// <returns>C^[Zv^[</returns>
        protected virtual IMethodInterceptor GetMethodInterceptor(
            S2DaoAttribute daoAttr)
        {
            Type settingType = daoAttr.DaoSettingType;
            if (settingType == null)
            {
                // gpHandlerw肳ĂȂꍇ
                // OX[
                throw new QuillApplicationException("EQLL0013");
            }

            IDaoSetting daoSetting = (IDaoSetting)ComponentUtil.GetComponent(
                container, settingType);

            if (daoSetting.IsNeedSetup())
            {
                //  DataSource̎擾
                IDataSource dataSource = (IDataSource)ComponentUtil.GetComponent(
                    container, typeof(SelectableDataSourceProxyWithDictionary));
                //  S2DaoInterceptor̐ݒ
                daoSetting.Setup(dataSource);
            }

            if (daoSetting.DaoInterceptor == null)
            {
                //  InterceptorĂȂꍇ͗OƂ
                throw new QuillApplicationException("EQLL0023");
            }
            return daoSetting.DaoInterceptor;
        }
    }
}
